@@ -92,6 +92,8 @@ typedef struct _IMUIMContext {
 
 static IMUIMContext context_list;
 static IMUIMContext *focused_context = NULL;
+static gboolean disable_focused_context = FALSE;
+
 static GObjectClass *parent_class;
 
 typedef struct _IMContextUIMClass 
@@ -434,6 +436,7 @@ focus_in(GtkIMContext *ic)
   IMUIMContext *cc;
 
   focused_context = uic;
+  disable_focused_context = FALSE;
 
   check_helper_connection();
 
@@ -460,9 +463,6 @@ focus_out(GtkIMContext *ic)
 {
   IMUIMContext *uic = IM_UIM_CONTEXT(ic);
 
-  if(focused_context == uic)
-    focused_context = NULL;
-
   uim_helper_client_focus_out(uic->uc);
 
   if(uic->cwin.top_win){
@@ -599,8 +599,10 @@ im_uim_finalize(GObject *obj)
   g_object_unref(uic->slave);
   parent_class->finalize (obj);
 
-  if(uic == focused_context)
+  if(uic == focused_context) {
     focused_context = NULL;
+    disable_focused_context = TRUE;
+  }
 }
 
 static void
@@ -1055,7 +1057,7 @@ im_uim_parse_helper_str(char *str)
 {
   gchar **lines;
   
-  if(focused_context) {
+  if(focused_context && !disable_focused_context) {
     if(strncmp("prop_list_get", str, 13) == 0){
       uim_prop_list_update(focused_context->uc);
     } else if(strncmp("prop_label_get", str, 14) == 0) {
@@ -1067,7 +1069,9 @@ im_uim_parse_helper_str(char *str)
 	g_strfreev(lines);
       }
     } else if(strncmp("focus_in", str, 8) == 0) {
-      /*      focused_context = NULL; */
+      disable_focused_context = TRUE;
+      /* We shouldn't do "focused_context = NULL" here, because some
+	 window manager has some focus related bugs. */
     }
   }
 }
