@@ -410,5 +410,32 @@
     ((("z" "u" "a" "n")) ("׬" "" "߬" "" "" ""))
     ((("z" "u" "i")) ("" "" "" "" "" "" "ީ"))
     ((("z" "u" "n")) ("" "" "" "" "ߤ"))
-    ((("z" "u" "o")) ("" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "")))
-  )
+    ((("z" "u" "o")) ("" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "" "" ""))
+    (((",")) (""))
+    (((".")) (""))
+    ((("?")) (""))
+    ((("!")) (""))
+    ((("\"")) (""))
+    ((("#")) (""))
+    ((("$")) (""))
+    ((("%")) (""))
+    ((("&")) (""))
+    ((("'")) (""))
+    ((("(")) (""))
+    (((")")) (""))
+    ((("=")) (""))
+    ((("^")) (""))
+    ((("\\")) (""))
+    ((("|")) (""))
+    ((("`")) (""))
+    ((("@")) (""))
+    ((("+")) (""))
+    (((";")) (""))
+    ((("*")) (""))
+    (((":")) (""))
+    ((("}")) (""))
+    ((("<")) (""))
+    (((">")) (""))
+    ((("/")) (""))
+    ((("_")) (""))
+  ))
