@@ -1,4 +1,5 @@
 ;;; anthy.scm: Anthy for uim.
+;;; charset: EUC-JP
 ;;;
 ;;; Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
 ;;;
@@ -32,7 +33,7 @@
 (require "japanese.scm")
 (require "generic-key.scm")
 ;;
-(define anthy-init-lib-ok? nil)
+(define anthy-init-lib-ok? '())
 ;; configs
 (define anthy-use-candidate-window? #t)
 (define anthy-candidate-op-count 1)
@@ -343,21 +344,6 @@
 	    (anthy-update-prop-label ac id))
 	  (anthy-commit-raw ac id)))))
 
-(define anthy-make-index-list-rec
-  (lambda (n)
-    (if (> n 0)
-	(cons 0
-	      (anthy-make-index-list-rec (- n 1)))
-	'())))
-
-(define anthy-make-index-list
-  (lambda (n old-lst)
-    (if (< n (length old-lst))
-	(truncate-list old-lst n 1)
-	(append old-lst
-		(anthy-make-index-list-rec (- n
-					      (length old-lst)))))))
-
 (define anthy-append-string
   (lambda (ac str)
     (and
@@ -370,104 +356,94 @@
 	 #t)
      (anthy-context-set-left-string!
       ac (cons str (anthy-context-left-string ac))))))
-    
 
 (define anthy-begin-conv
   (lambda (ac)
     (let* ((ac-id (anthy-context-ac-id ac))
 	   (rkc (anthy-context-rkc ac))
+	   (pending (rk-pending rkc))
 	   (kana (anthy-context-kana-mode ac))
 	   (last "")
-	   (res))
+	   (res (rk-push-key-last! rkc)))
       ;; get residual 'n'
-      (set! res (rk-push-key-last! rkc))
       (if res
-	  (anthy-append-string ac res))
+	  (begin
+	    (anthy-append-string ac res)
+	    (set! pending "")))
       ;;
       (anthy-lib-set-string
        ac-id
        (string-append
 	(anthy-make-left-string (anthy-context-left-string ac)  anthy-type-hiragana)
+	pending
 	(anthy-make-right-string (anthy-context-right-string ac) anthy-type-hiragana)))
       (anthy-context-set-index-list!
        ac
-       (anthy-make-index-list
+       (multi-segment-make-index-list
 	(anthy-lib-get-nr-segments ac-id)
-	nil))
+	'()))
       (anthy-context-set-state! ac #t)
       (anthy-context-set-cur-seg! ac 0)
       (rk-flush (anthy-context-rkc ac)))))
 
 (define anthy-proc-input-state-no-preedit
   (lambda (ac id key key-state)
-    (let
-	((rkc (anthy-context-rkc ac))
-	 (direct (ja-direct (charcode->string key))))
-      (and
-
-       (if (anthy-wide-latin-key? key key-state)
-	   (begin
-	     (anthy-flush ac)
-	     (anthy-context-set-on! ac #f)
-	     (anthy-context-set-wide-latin! ac #t)
-	     (anthy-update-mode ac id)
-	     (anthy-update-prop-label ac id)
-	     #f)
-	   #t)
-       (if (anthy-latin-key? key key-state)
+    (let ((rkc (anthy-context-rkc ac))
+	  (direct (ja-direct (charcode->string key))))
+      (cond
+       ((anthy-wide-latin-key? key key-state)
+	(begin
+	  (anthy-flush ac)
+	  (anthy-context-set-on! ac #f)
+	  (anthy-context-set-wide-latin! ac #t)
+	  (anthy-update-mode ac id)
+	  (anthy-update-prop-label ac id)))
+	  
+       ((anthy-latin-key? key key-state)
 	   (begin
 	     (anthy-flush ac)
 	     (anthy-context-set-on! ac #f)
 	     (anthy-context-set-wide-latin! ac #f)
 	     (anthy-update-mode ac id)
-	     (anthy-update-prop-label ac id)
-	     #f)
-	   #t)
-       (if (anthy-backspace-key? key key-state)
-	   (begin
-	     (anthy-commit-raw ac id)
-	     #f)
-	   #t)
-       (if (anthy-delete-key? key key-state)
-	   (begin
-	     (anthy-commit-raw ac id)
-	     #f)
-	   #t)
-       (if (anthy-hankaku-kana-key? key key-state)
+	     (anthy-update-prop-label ac id)))
+
+       ((anthy-backspace-key? key key-state)
+	(anthy-commit-raw ac id))
+
+       ((anthy-delete-key? key key-state)
+	(anthy-commit-raw ac id))
+       
+       ((anthy-hankaku-kana-key? key key-state)
 	   (begin 
 	     (anthy-context-set-kana-mode! ac anthy-type-hankana)
 	     (anthy-update-mode ac id)
-	     (anthy-update-prop-label ac id)
-	     #f)
-	   #t)
-       (if (anthy-kana-toggle-key? key key-state)
-	   (begin 
-	     (anthy-context-kana-toggle ac)
-	     (anthy-update-mode ac id)
-	     (anthy-update-prop-label ac id)
-	     #f)
-	   #t)
+	     (anthy-update-prop-label ac id)))
+
+       ((anthy-kana-toggle-key? key key-state)
+	(begin 
+	  (anthy-context-kana-toggle ac)
+	  (anthy-update-mode ac id)
+	  (anthy-update-prop-label ac id)))
+
        ;; modifiers (except shift) => ignore
-       (if (and (modifier-key-mask key-state)
-		(not (shift-key-mask key-state)))
-	   (begin
-	     (anthy-commit-raw ac id)
-	     #f)
-	   #t)
+       ((and (modifier-key-mask key-state)
+	     (not (shift-key-mask key-state)))
+	(anthy-commit-raw ac id))
+       
        ;; direct key => commit
-       (if direct
-	   (begin
-	     (im-commit id direct)
-	     #f)
-	   #t)
-       (set! key-str (charcode->string (to-lower-char key)))
-       (if (string-find (rk-expect rkc) key-str)
-	   (let ((res (rk-push-key! rkc key-str)))
-	     (if res
-		 (anthy-append-string ac res))
-	     #f)
-	   #t)
-       (anthy-commit-raw ac id)))))
+       (direct
+	(im-commit id direct))
+
+       (else
+	(begin
+	  (set! key-str (charcode->string (to-lower-char key)))
+	  (string-find (rk-expect rkc) key-str)
+	  (let ((res (rk-push-key! rkc key-str)))
+	    (if res
+		(anthy-append-string ac res)
+		(if (not (rk-pending rkc)
+			 (anthy-commit-raw ac id)))))))
+       ))))
 
 (define anthy-has-preedit?
   (lambda (ac)
@@ -480,167 +456,136 @@
   (lambda (ac id key key-state)
     (let ((rkc (anthy-context-rkc ac))
 	  (kana (anthy-context-kana-mode ac)))
-      (and
+      (cond
+       
        ;; begin conversion
-       (if (and
-	    (anthy-begin-conv-key? key key-state)
-	    (number? (anthy-context-ac-id ac)))
-	   (begin
-	     (anthy-begin-conv ac)
-	     #f)
-	   #t)
+       ((and (anthy-begin-conv-key? key key-state)
+	     (number? (anthy-context-ac-id ac)))
+	(anthy-begin-conv ac))
+       
        ;; backspace
-       (if (anthy-backspace-key? key key-state)
-	   (begin
-	     (if (not (rk-backspace rkc))
-		 (if (anthy-context-left-string ac)
-		     (anthy-context-set-left-string!
-		      ac
-		      (cdr (anthy-context-left-string ac)))))
-	     #f)
-	   #t)
+       ((anthy-backspace-key? key key-state)
+	(if (not (rk-backspace rkc))
+	    (if (anthy-context-left-string ac)
+		(anthy-context-set-left-string!
+		 ac
+		 (cdr (anthy-context-left-string ac))))))
+       
        ;; delete
-       (if (anthy-delete-key? key key-state)
-	   (begin
-	     (if (not (rk-delete rkc))
-		 (if (anthy-context-right-string ac)
-		     (anthy-context-set-right-string!
-		      ac
-		      (cdr (anthy-context-right-string ac)))))
-	     #f)
-	   #t)
+       ((anthy-delete-key? key key-state)
+	(if (not (rk-delete rkc))
+	    (if (anthy-context-right-string ac)
+		(anthy-context-set-right-string!
+		 ac
+		 (cdr (anthy-context-right-string ac))))))
+
        ;; kill
-       (if (anthy-kill-key? key key-state)
-	   (begin
-	     (anthy-context-set-right-string! ac ())
-	     #f)
-	   #t)
+       ((anthy-kill-key? key key-state)
+	(anthy-context-set-right-string! ac ()))
+       
        ;; kill-backward
-       (if (anthy-kill-backward-key? key key-state)
-	   (begin
-	     (rk-flush rkc)
-	     (anthy-context-set-left-string! ac ())
-	     #f)
-	   #t)
+       ((anthy-kill-backward-key? key key-state)
+	(begin
+	  (rk-flush rkc)
+	  (anthy-context-set-left-string! ac ())))
+       
        ;; Ҥ餬ʥ⡼ɤǥʤꤹ
-       (if (anthy-commit-as-opposite-kana-key? key key-state)
-	   (begin
-	     (im-commit
-	      id
-	      (string-append
-	       (anthy-make-left-string (anthy-context-left-string ac)
-				       (anthy-opposite-kana kana))
-	       (anthy-make-right-string (anthy-context-right-string ac)
-					(anthy-opposite-kana kana))))
-	     (anthy-flush ac)
-	     #f)
-	   #t)
+       ((anthy-commit-as-opposite-kana-key? key key-state)
+	(begin
+	  (im-commit
+	   id
+	   (string-append
+	    (anthy-make-left-string (anthy-context-left-string ac)
+				    (anthy-opposite-kana kana))
+	    (anthy-make-right-string (anthy-context-right-string ac)
+				     (anthy-opposite-kana kana))))
+	  (anthy-flush ac)))
+
        ;; ߤΤʤ塢Ҥ餬/ʥ⡼ɤڤ괹
-       (if (anthy-kana-toggle-key? key key-state)
-	   (begin
-	     (im-commit
-	      id
-	      (string-append
-	       (anthy-make-left-string (anthy-context-left-string ac)
-				       kana)
-	       (anthy-make-right-string (anthy-context-right-string ac)
-					kana)))
-	     (anthy-flush ac)
-	     (anthy-context-kana-toggle ac)
-	     (anthy-update-mode ac id)
-	     (anthy-update-prop-label ac id)
-	     #f)
-	   #t)
+       ((anthy-kana-toggle-key? key key-state)
+	(begin
+	  (im-commit
+	   id
+	   (string-append
+	    (anthy-make-left-string (anthy-context-left-string ac)
+				    kana)
+	    (anthy-make-right-string (anthy-context-right-string ac)
+				     kana)))
+	  (anthy-flush ac)
+	  (anthy-context-kana-toggle ac)
+	  (anthy-update-mode ac id)
+	  (anthy-update-prop-label ac id)))
+
        ;; cancel
-       (if (anthy-cancel-key? key key-state)
-	   (begin
-	     (anthy-flush ac)
-	     #f)
-	   #t)
+       ((anthy-cancel-key? key key-state)
+	(anthy-flush ac))
+
        ;; commit
-       (if (anthy-commit-key? key key-state)
-	   (begin
-	     (im-commit
-	      id
-	      (string-append
-	       (anthy-make-left-string (anthy-context-left-string ac) kana)
-	       (rk-pending rkc)
-	       (anthy-make-right-string (anthy-context-right-string ac) kana)))
-	     (anthy-flush ac)
-	     #f)
-	   #t)
+       ((anthy-commit-key? key key-state)
+	(begin
+	  (im-commit
+	   id
+	   (string-append
+	    (anthy-make-left-string (anthy-context-left-string ac) kana)
+	    (rk-pending rkc)
+	    (anthy-make-right-string (anthy-context-right-string ac) kana)))
+	  (anthy-flush ac)))
+
        ;; left
-       (if (anthy-go-left-key? key key-state)
-	   (begin
-	     (if (anthy-context-left-string ac)
-		 (let ((c (car (anthy-context-left-string ac))))
-		   (anthy-context-set-left-string!
-		    ac (cdr (anthy-context-left-string ac)))
-		   (anthy-context-set-right-string! 
-		    ac
-		    (cons c (anthy-context-right-string ac)))))
-	     #f)
-	   #t)
+       ((anthy-go-left-key? key key-state)
+	(if (anthy-context-left-string ac)
+	    (let ((c (car (anthy-context-left-string ac))))
+	      (anthy-context-set-left-string!
+	       ac (cdr (anthy-context-left-string ac)))
+	      (anthy-context-set-right-string! 
+	       ac
+	       (cons c (anthy-context-right-string ac))))))
+
        ;; right
-       (if (anthy-go-right-key? key key-state)
-	   (begin
-	     (if (anthy-context-right-string ac)
-		 (let ((c (car (anthy-context-right-string ac))))
-		   (anthy-context-set-right-string!
-		    ac (cdr (anthy-context-right-string ac)))
-		   (anthy-append-string ac c)))
-	     #f)
-	   #t)
+       ((anthy-go-right-key? key key-state)
+	(if (anthy-context-right-string ac)
+	    (let ((c (car (anthy-context-right-string ac))))
+	      (anthy-context-set-right-string!
+	       ac (cdr (anthy-context-right-string ac)))
+	      (anthy-append-string ac c))))
+
        ;; beginning-of-preedit
-       (if (anthy-beginning-of-preedit-key? key key-state)
-	   (begin
-	     (if (anthy-context-left-string ac)
-		 (begin
-		   (anthy-context-set-right-string!
-		    ac
-		    (append
-		     (reverse (anthy-context-left-string ac))
-		     (anthy-context-right-string ac)))
-		   (anthy-context-set-left-string! ac ())))
-	     #f)
-	   #t)
+       ((anthy-beginning-of-preedit-key? key key-state)
+	(if (anthy-context-left-string ac)
+	    (begin
+	      (anthy-context-set-right-string!
+	       ac
+	       (append
+		(reverse (anthy-context-left-string ac))
+		(anthy-context-right-string ac)))
+	      (anthy-context-set-left-string! ac ()))))
+
        ;; end-of-preedit
-       (if (anthy-end-of-preedit-key? key key-state)
-	   (begin
-	     (if (anthy-context-right-string ac)
-		 (begin
-		   (anthy-context-set-left-string!
-		    ac
-		    (append
-		     (reverse (anthy-context-right-string ac))
-		     (anthy-context-left-string ac)))
-		   (anthy-context-set-right-string! ac ())))
+       ((anthy-end-of-preedit-key? key key-state)
+	(if (anthy-context-right-string ac)
+	    (begin
+	      (anthy-context-set-left-string!
+	       ac
+	       (append
+		(reverse (anthy-context-right-string ac))
+		(anthy-context-left-string ac)))
+	      (anthy-context-set-right-string! ac ()))))
 ;		   (rk-flush rkc)))
-	     #f)
-	   #t)
+
        ;; modifiers (except shift) => ignore
-       (if (and (modifier-key-mask key-state)
-		(not (shift-key-mask key-state)))
-	   (begin
-	     (anthy-commit-raw ac id)
-	     #f)
-	   #t)
-  ;    (if (symbol? key)
-;	   (begin
-;	     (anthy-commit-raw ac id)
-;	     #f)
-;	   #t)
-;       (set! key (to-lower-char key))
-       (let* ((key-str (charcode->string key))
-	      (pend (rk-pending rkc))
-	      (res (rk-push-key! rkc key-str)))
-	 (if (and res (not (string-equal? (car res) "")))
-	     (anthy-append-string ac res)
-	     (if using-kana-table?
-	     (begin
-	       (anthy-append-string ac (list pend "" ""))
-;	       (anthy-append-string ac (list (rk-pending rkc) "" "")
-))))))))
+       ((and (modifier-key-mask key-state)
+	     (not (shift-key-mask key-state)))
+	(anthy-commit-raw ac id))
+
+       (else	
+	(let* ((key-str (charcode->string key))
+	       (pend (rk-pending rkc))
+	       (res (rk-push-key! rkc key-str)))
+	  (if (and res (not (string-equal? (car res) "")))
+	      (anthy-append-string ac res)
+	      (if using-kana-table?
+		  (anthy-append-string ac (list pend "" "")))
+	      )))))))
 
 (define anthy-proc-input-state
   (lambda (ac id key key-state)
@@ -728,7 +673,7 @@
 	     ac-id idx (nth idx (anthy-context-index-list ac)))
 	    (anthy-commit-string ac
 				 (+ idx 1) nseg))
-	  nil))))
+	  '()))))
 
 (define anthy-move-segment
   (lambda (ac dir)
@@ -783,7 +728,7 @@
        ac-id (anthy-context-cur-seg ac) cnt)
       (anthy-context-set-index-list!
        ac
-       (anthy-make-index-list
+       (multi-segment-make-index-list
 	(anthy-lib-get-nr-segments ac-id)
 	(truncate-list
 	 (anthy-context-index-list ac)
@@ -792,87 +737,66 @@
 
 (define anthy-proc-compose-state
   (lambda (ac id key key-state)
-    (and
-     (if (anthy-commit-key? key key-state)
-	 (begin
-	   (anthy-do-commit ac id)
-	   #f)
-	 #t)
-     (if (anthy-extend-segment-key? key key-state)
-	 (begin
-	   (anthy-resize-segment ac id 1)
-	   #f)
-	 #t)
-     (if (anthy-shrink-segment-key? key key-state)
-	 (begin
-	   (anthy-resize-segment ac id -1)
-	   #f)
-	 #t)
-     (if (anthy-next-segment-key? key key-state)
-	 (begin
-	   (anthy-move-segment ac 1)
-	   (anthy-reset-candidate-window ac id)
-	   #f)
-	 #t)
-     (if (anthy-prev-segment-key? key key-state)
-	 (begin
-	   (anthy-move-segment ac -1)
-	   (anthy-reset-candidate-window ac id)
-	   #f)
-	 #t)
-     (if (anthy-beginning-of-preedit-key? key key-state)
-	 (begin
-	   (anthy-context-set-cur-seg! ac 0)
-	   (anthy-reset-candidate-window ac id)
-	   #f)
-	 #t)
-     (if (anthy-end-of-preedit-key? key key-state)
-	 (let ((nseg (length (anthy-context-index-list ac))))
-	   (anthy-context-set-cur-seg! ac (- nseg 1))
-	   (anthy-reset-candidate-window ac id)
-	   #f)
-	 #t)
-     (if (anthy-backspace-key? key key-state)
-	 (begin
-	   (anthy-context-set-state! ac #f)
-	   (anthy-reset-candidate-window ac id)
-	   #f)
-	 #t)
-     (if (anthy-next-candidate-key? key key-state)
-	 (begin
-	   (anthy-move-candidate ac id 1)
-	   #f)
-	 #t)
-     (if (anthy-prev-candidate-key? key key-state)
-	 (begin
-	   (anthy-move-candidate ac id -1)
-	   #f)
-	 #t)
-     (if (anthy-cancel-key? key key-state)
-	 (begin
-	   (anthy-context-set-state! ac #f)
-	   (anthy-reset-candidate-window ac id)
-	   #f)
-	 #t)
-     (if (anthy-prev-page-key? key key-state)
-	 (begin
-	   (im-shift-page-candidate id #f)
-	   #f)
-	 #t)
-     (if (anthy-next-page-key? key key-state)
-	 (begin
-	   (im-shift-page-candidate id #t)
-	   #f)
-	 #t)
-     (if (and (modifier-key-mask key-state)
-	      (not (shift-key-mask key-state)))
-	 #f
-	 #t)
-     (begin
-       (anthy-do-commit ac id)
-       (anthy-proc-input-state ac id key key-state)))
-    ()))
+    (cond
+     ((anthy-commit-key? key key-state)
+      (anthy-do-commit ac id))
+     
+     ((anthy-extend-segment-key? key key-state)
+      (anthy-resize-segment ac id 1))
+     
+     ((anthy-shrink-segment-key? key key-state)
+      (anthy-resize-segment ac id -1))
+     
+     ((anthy-next-segment-key? key key-state)
+      (begin
+	(anthy-move-segment ac 1)
+	(anthy-reset-candidate-window ac id)))
+     
+     ((anthy-prev-segment-key? key key-state)
+      (begin
+	(anthy-move-segment ac -1)
+	(anthy-reset-candidate-window ac id)))
 
+     ((anthy-beginning-of-preedit-key? key key-state)
+      (begin
+	(anthy-context-set-cur-seg! ac 0)
+	(anthy-reset-candidate-window ac id)))
+
+     ((anthy-end-of-preedit-key? key key-state)
+      (let ((nseg (length (anthy-context-index-list ac))))
+	(anthy-context-set-cur-seg! ac (- nseg 1))
+	(anthy-reset-candidate-window ac id)))
+
+     ((anthy-backspace-key? key key-state)
+      (begin
+	(anthy-context-set-state! ac #f)
+	(anthy-reset-candidate-window ac id)))
+
+     ((anthy-next-candidate-key? key key-state)
+      (anthy-move-candidate ac id 1))
+
+     ((anthy-prev-candidate-key? key key-state)
+      (anthy-move-candidate ac id -1))
+
+     ((anthy-cancel-key? key key-state)
+      (begin
+	(anthy-context-set-state! ac #f)
+	(anthy-reset-candidate-window ac id)))
+
+     ((anthy-prev-page-key? key key-state)
+	(im-shift-page-candidate id #f))
+
+     ((anthy-next-page-key? key key-state)
+	(im-shift-page-candidate id #t))
+
+     ((modifier-key-mask key-state)
+      ())
+
+     (else
+      (begin
+	(anthy-do-commit ac id)
+	(anthy-proc-input-state ac id key key-state))))
+    ))
 
 (define anthy-proc-wide-latin
   (lambda (c key key-state)
@@ -899,8 +823,7 @@
   (lambda (id key key-state)
     (let* ((c (find-context id))
 	   (ac (context-data c)))
-      (if (and (number? key)
-	       (< key 32))
+      (if (control-char? key)
 	  (im-commit-raw id)
 	  (if (anthy-context-on ac)
 	      (if (anthy-context-state ac)
@@ -917,11 +840,11 @@
   (lambda (id key key-state)
     (let* ((c (find-context id))
 	   (ac (context-data c)))
-      (if (and
-	   (not (anthy-context-on ac))
-	   (not (anthy-context-wide-latin ac))
-	   (and (number? key) (< key 32)))
-	  ;; don't eat key release event for apps
+      (if (or (control-char? key)
+	      (and
+	       (not (anthy-context-on ac))
+	       (not (anthy-context-wide-latin ac))))
+	  ;; don't discard key release event for apps
 	  (anthy-commit-raw ac id)))))
 
 (define anthy-reset-handler
@@ -1050,7 +973,7 @@
  'anthy
  "ja"
  "EUC-JP"
- nil
+ '()
  anthy-init-handler
  anthy-release-handler
  anthy-mode-handler
