@@ -47,12 +47,12 @@
 ;;; Key definition
 (define-key canna-on-key? "<Control>\\")
 (define-key canna-off-key? "<Control>\\")
-(define-key canna-latin-key "<Control>\\")
+(define-key canna-latin-key? "<Control>\\")
 ;(define-key canna-latin-key '("l" generic-on-key?))
 (define-key canna-wide-latin-key? "L")
-;(define-key canna-hankaku-kana-key? '("<Control>q" "<Control>Q"))
+(define-key canna-hankaku-kana-key? '("<Control>q" "<Control>Q"))
 (define-key canna-kana-toggle-key? "q")
-;(define-key canna-commit-as-opposite-kana-key? "Q")
+(define-key canna-commit-as-opposite-kana-key? "Q")
 (define-key canna-begin-conv-key? '(" " generic-on-key?))
 (define-key canna-commit-key? 'generic-commit-key?)
 (define-key canna-extend-segment-key? '("<Control>o" "<Control>O" "<Shift>right"))
@@ -149,7 +149,7 @@
 (define (canna-context-new)
   (let ((c '())
 	(rkc (rk-context-new ja-rk-rule #t #f)))
-    (set! c (copy-list '(() () () () () () () () () () () #f #t #f)))
+    (set! c (copy-list '(() () () () () () () () () () () 0 #t #f)))
     (canna-context-set-cc-id! c (if canna-init-lib-ok?
 				    (canna-lib-alloc-context) ()))
     (canna-context-set-rkc! c rkc)
@@ -161,45 +161,41 @@
   (im-commit-raw id)
   (canna-context-set-commit-raw! cc #t))
 
+(define canna-opposite-kana
+  (lambda (kana)
+    (cond
+     ((= kana multi-segment-type-hiragana)
+      multi-segment-type-katakana)
+     ((= kana multi-segment-type-katakana)
+      multi-segment-type-hiragana)
+     ((= kana multi-segment-type-hankana)
+      multi-segment-type-hiragana))))
+
 (define (canna-context-kana-toggle cc)
-  (canna-context-set-kana-mode! cc (not (canna-context-kana-mode cc))))
-
-(define (canna-make-string sl dir kana)
-  (if sl
-      (if dir
-	  (string-append (canna-make-string (cdr sl) dir kana)
-			 (if kana
-			     (cdar sl)
-			     (caar sl)))
-	  (string-append (if kana
-			     (cdar sl)
-			     (caar sl))
-			 (canna-make-string (cdr sl) dir kana)))
-      ""))
-
-(define (canna-make-left-string sl kana)
-  (canna-make-string sl #t kana))
-
-(define (canna-make-right-string sl kana)
-  (canna-make-string sl #f kana))
+  (let* ((kana (canna-context-kana-mode cc))
+	 (opposite-kana (canna-opposite-kana kana)))
+    (canna-context-set-kana-mode! cc opposite-kana)))
 
 (define (canna-mode-handler id mode)
   (let* ((c (find-context id))
 	 (cc (context-date c)))
     (canna-flush cc)
     (cond
-     ((= mode 0)
+     ((= mode multi-segment-mode-direct)
       (canna-context-set-on! cc #f)
       (canna-context-set-wide-latin! cc #f))
-     ((= mode 1)
+     ((= mode multi-segment-mode-hiragana)
       (canna-context-set-on! cc #t)
-      (canna-context-set-kana-mode! cc #f))
-     ((= mode 2)
+      (canna-context-set-kana-mode! cc multi-segment-type-hiragana))
+     ((= mode multi-segment-mode-katakana)
       (canna-context-set-on! cc #t)
-      (canna-context-set-kana-mode! cc #t))
-     ((= mode 3)
+      (canna-context-set-kana-mode! cc multi-segment-type-katakana))
+     ((= mode multi-segment-mode-wide-latin)
       (canna-context-set-on! cc #f)
-      (canna-context-set-wide-latin! cc #t)))
+      (canna-context-set-wide-latin! cc #t))
+     ((= mode multi-segment-mode-hankana)
+      (canna-context-set-on! cc #t)
+      (canna-context-set-kana-mode! cc multi-segment-type-hankana)))
     (canna-update-preedit cc id))
   ())
 
@@ -227,19 +223,6 @@
 	(im-clear-preedit id)
 	(im-update-preedit id))))
 
-(define (canna-make-index-list-rec n)
-  (if (> n 0)
-      (cons 0
-	    (canna-make-index-list-rec (- n 1)))
-      '()))
-
-(define (canna-make-index-list n old-lst)
-  (if (< n (length old-lst))
-      (truncate-list old-lst n 1)
-      (append old-lst
-	      (canna-make-index-list-rec (- n
-					    (length old-lst))))))
-
 (define (canna-append-string cc str)
   (and str
        (if (not (string? (car str)))
@@ -263,17 +246,18 @@
 
     (canna-context-set-index-list!
      cc
-     (canna-make-index-list
+     (multi-segment-make-index-list
       (canna-lib-begin-conversion
        cc-id
        (string-append
-	(canna-make-left-string (canna-context-left-string cc) #f)
-	(canna-make-right-string (canna-context-right-string cc) #f))) nil))
+	(multi-segment-make-left-string (canna-context-left-string cc)
+					multi-segment-type-hiragana)
+	(multi-segment-make-right-string (canna-context-right-string cc)
+					 multi-segment-type-hiragana))) nil))
     (canna-context-set-state! cc #t)
     (canna-context-set-cur-seg! cc 0)
     (rk-flush (canna-context-rkc cc))))
 
-
 (define (canna-proc-input-state-no-preedit cc id key key-state)
   (let
       ((rkc (canna-context-rkc cc))
@@ -307,6 +291,13 @@
 	   (im-commit-raw id)
 	   #f)
 	 #t)
+     (if (canna-hankaku-kana-key? key key-state)
+	 (begin
+	   (canna-context-set-kana-mode! cc multi-segment-type-hankana)
+	   (canna-update-mode cc id)
+	   (canna-update-prop-label cc id)
+	   #f)
+	 #t)
      (if (canna-kana-toggle-key? key key-state)
 	 (begin
 	   (canna-context-kana-toggle cc)
@@ -377,16 +368,32 @@
 	   #f)
 	 #t)
        ;; Ҥ餬ʥ⡼ɤǥʤꤹ
+      (if (canna-commit-as-opposite-kana-key? key key-state)
+	   (begin
+	     (im-commit
+	      id
+	      (string-append
+	       (multi-segment-make-left-string (canna-context-left-string cc)
+					       (canna-opposite-kana kana))
+	       (multi-segment-make-right-string (canna-context-right-string cc)
+						(canna-opposite-kana kana))))
+	     (canna-flush cc)
+	     #f)
+	   #t)
+      ;; ߤΤʤ塢Ҥ餬/ʥ⡼ɤڤ괹
      (if (canna-kana-toggle-key? key key-state)
 	 (begin
 	   (im-commit
 	    id
 	    (string-append
-	     (canna-make-left-string (canna-context-left-string cc)
-				     (not kana))
-	     (canna-make-right-string (canna-context-right-string cc)
-				      (not kana))))
+	     (multi-segment-make-left-string (canna-context-left-string cc)
+					     kana)
+	     (multi-semgnet-make-right-string (canna-context-right-string cc)
+					      kana)))
 	   (canna-flush cc)
+	   (canna-context-kana-toggle cc)
+	   (canna-update-mode cc id)
+	   (canna-update-prop-label cc id)
 	   #f)
 	 #t)
        ;; cancel
@@ -401,8 +408,11 @@
 	   (im-commit
 	    id
 	    (string-append
-	     (canna-make-left-string (canna-context-left-string cc) kana)
-	     (canna-make-right-string (canna-context-right-string cc) kana)))
+	     (multi-segment-make-left-string (canna-context-left-string cc)
+					     kana)
+	     (rk-pending rkc)
+	     (multi-segment-make-right-string (canna-context-right-string cc)
+					      kana)))
 	   (canna-flush cc)
 	   #f)
 	 #t)
@@ -437,12 +447,21 @@
 	   (im-commit-raw id)
 	   #f)
 	 #t)
-     (set! key (to-lower-char key))
-     (let ((res)
-	   (key-str (charcode->string key)))
-       (set! res (rk-push-key! rkc key-str))
-       (if res
-	   (canna-append-string cc res))))))
+     (let* ((key-str (charcode->string key))
+	    (pend (rk-pending rkc))
+	    (res (rk-push-key! rkc key-str)))
+       (if (and res (not (string-equal? (car res) "")))
+	   (canna-append-string cc res)
+	   (if using-kana-table?
+	       (begin
+		 (canna-append-string cc (list pend "" ""))
+;     (set! key (to-lower-char key))
+;     (let ((res)
+;	   (key-str (charcode->string key)))
+;       (set! res (rk-push-key! rkc key-str))
+;       (if res
+;	   (canna-append-string cc res))))))
+)))))))
 
 (define (canna-proc-input-state cc id key key-state)
   (if (canna-has-preedit? cc)
@@ -484,14 +503,14 @@
     (im-clear-preedit id)
     (im-pushback-preedit
      id preedit-underline
-     (canna-make-left-string (canna-context-left-string cc) kana))
+     (multi-segment-make-left-string (canna-context-left-string cc) kana))
     (im-pushback-preedit id preedit-underline
 			 (rk-pending rkc))
     (if (canna-has-preedit? cc)
 	(im-pushback-preedit id preedit-cursor ""))
     (im-pushback-preedit
      id preedit-underline
-     (canna-make-right-string (canna-context-right-string cc) kana))
+     (multi-segment-make-right-string (canna-context-right-string cc) kana))
     (im-update-preedit id)))
 
 (define (canna-get-commit-string cc idx nseg)
@@ -533,8 +552,10 @@
     (im-pushback-mode-list id "Ҥ餬")
     (im-pushback-mode-list id "")
     (im-pushback-mode-list id "ѱѿ")
+    (im-pushback-mode-list id "Ⱦѥ")
     (im-update-mode-list id)
     (im-update-mode id 0)
+    (canna-update-prop-list id)
     ()))
 
 (define (canna-release-handler id)
@@ -548,18 +569,21 @@
 	 (cc (context-data c)))
     (canna-flush cc)
     (cond  ; `case' is not supported by uim
-     ((= mode 0)  ; 'direct'
+     ((= mode multi-segment-mode-direct)  ; 'direct'
       (canna-context-set-on! cc #f)
       (canna-context-set-wide-latin! cc #f))
-     ((= mode 1)  ; 'hiragana'
+     ((= mode multi-segment-mode-hiragana)  ; 'hiragana'
       (canna-context-set-on! cc #t)
-      (canna-context-set-kana-mode! cc #f))
-     ((= mode 2)  ; 'katakana'
+      (canna-context-set-kana-mode! cc multi-segment-type-hiragana))
+     ((= mode multi-segment-mode-katakana)  ; 'katakana'
       (canna-context-set-on! cc #t)
-      (canna-context-set-kana-mode! cc #t))
-     ((= mode 3)  ; 'wide-latin'
+      (canna-context-set-kana-mode! cc multi-segment-type-katakana))
+     ((= mode multi-segment-mode-wide-latin)  ; 'wide-latin'
       (canna-context-set-on! cc #f)
-      (canna-context-set-wide-latin! cc #t))) 
+      (canna-context-set-wide-latin! cc #t))
+     ((= mode multi-segment-mode-hankana)
+      (canna-context-set-on! cc #t)
+      (canna-context-set-kana-mode! cc multi-segment-type-hankana)))  
     (canna-update-preedit cc id))
   ())
 
@@ -612,7 +636,7 @@
        cc-id (canna-context-cur-seg cc) cnt)
       (canna-context-set-index-list!
        cc
-       (canna-make-index-list
+       (multi-segment-make-index-list
 	(canna-lib-get-nr-segments cc-id)
 	(truncate-list
 	 (canna-context-index-list cc)
@@ -695,7 +719,7 @@
       (canna-update-prop-label cc id))
      ((and (modifier-key-mask key-state)
 	   (not (shift-key-mask key-state)))
-      (im-commit-raw id))
+      (canna-commit-raw id))
      (w
       (im-commit id w))
      (else
@@ -705,22 +729,25 @@
 (define (canna-press-key-handler id key key-state)
   (let* ((c (find-context id))
 	 (cc (context-data c)))
-    (if (canna-context-on cc)
-	(if (canna-context-state cc)
-	    (canna-proc-compose-state cc id key key-state)
-	    (canna-proc-input-state cc id key key-state))
-	(if (canna-context-wide-latin cc)
-	    (canna-proc-wide-latin c key key-state)
-	    (canna-proc-raw-state c key key-state)))
+    (if (control-char? key)
+	(im-commit-raw id)
+	(if (canna-context-on cc)
+	    (if (canna-context-state cc)
+		(canna-proc-compose-state cc id key key-state)
+		(canna-proc-input-state cc id key key-state))
+	    (if (canna-context-wide-latin cc)
+		(canna-proc-wide-latin c key key-state)
+		(canna-proc-raw-state c key key-state))))
     (canna-update-preedit cc id)))
 
 ;;;
 (define (canna-release-key-handler id key key-state)
   (let* ((c (find-context id))
 	 (cc (context-data c)))
-    (if (and
-	 (not (canna-context-on cc))
-	 (not (canna-context-wide-latin cc)))
+    (if (or (control-char? key)
+	    (and
+	     (not (canna-context-on cc))
+	     (not (canna-context-wide-latin cc))))
 	(canna-commit-raw cc id))))
 ;;;
 (define (canna-reset-handler id)
@@ -753,26 +780,36 @@
 	 (cc (context-data c)))
     (canna-flush cc)
     (canna-update-preedit cc id)
-    (if (string-equal? message
-		       "prop_canna_hiragana")
-	(begin
-	  (canna-context-set-on! cc #t)
-	  (canna-context-set-kana-mode! cc #t)))
-    (if (string-equal? message
-		       "prop_canna_katakana")
-	(begin
-	  (canna-context-set-on! cc #t)
-	  (canna-context-set-kana-mode! cc #f)))
-    (if (string-equal? message
-		       "prop_canna_direct")
+    (cond
+     ((string-equal? message
+		     "prop_canna_hiragana")
+      (begin
+	(canna-context-set-on! cc #t)
+	(canna-context-set-kana-mode! cc multi-segment-type-hiragana)))
+     ((string-equal? message
+		     "prop_canna_katakana")
+      (begin
+	(canna-context-set-on! cc #t)
+	(canna-context-set-kana-mode! cc multi-segment-type-katakana)))
+     ((string-equal? message
+		     "prop_canna_hankana")
+      (begin
+	(canna-context-set-on! cc #t)
+	(canna-context-set-kana-mode! cc multi-segment-type-hankana)))
+     ((string-equal? message
+		     "prop_canna_direct")
 	(begin
 	  (canna-context-set-on! cc #f)
 	  (canna-context-set-wide-latin! cc #f)))
-    (if (string-equal? message
-		       "prop_canna_zenkaku")
-	(begin
-	  (canna-context-set-on! cc #f)
-	  (canna-context-set-wide-latin! cc #t)))
+     ((string-equal? message
+		     "prop_canna_zenkaku")
+      (begin
+	(canna-context-set-on! cc #f)
+	(canna-context-set-wide-latin! cc #t)))
+     ((string-equal? message
+		     "prop_canna_kana")
+      (begin
+	(load-kana-table))))
     (canna-update-mode cc id)
     (canna-update-prop-label cc id)))
 ;;;
@@ -790,12 +827,17 @@
 )))
 
 (define (canna-update-prop-label cc id)
-  (let ((str ""))
+  (let ((str "")
+	(kana (canna-context-kana-mode cc)))
     (set! str
 	  (if (canna-context-on cc)
-	      (if (canna-context-kana-mode cc)
-		  "\t\n"
-		  "\tҤ餬\n")
+	      (cond
+	       ((= kana multi-segment-type-hiragana)
+		"\tҤ餬\n")
+	       ((= kana multi-segment-type-katakana)
+		"\t\n")
+	       ((= kana multi-segment-type-hankana)
+		"\tȾѥ\n"))
 	      (if (canna-context-wide-latin cc)
 		  "\tѱѿ\n"
 		  "a\tľ\n")))
@@ -804,12 +846,17 @@
 
 (define (canna-update-mode cc id)
   (if (canna-context-on cc)
-      (if (canna-context-kana-mode cc)
-	  (im-update-mode id 2)
-	  (im-update-mode id 1))
+      (let ((kana (canna-context-kana-mode cc)))
+	(cond
+	 ((= kana multi-segment-type-hiragana)
+	  (im-update-mode id multi-segment-mode-hiragana))
+	 ((= kana multi-segment-type-katakana)
+	  (im-update-mode id multi-segment-mode-katakana))
+	 ((= kana multi-segment-type-hankana)
+	  (im-update-mode id multi-segment-mode-hankana))))
       (if (canna-context-wide-latin cc)
-	  (im-update-mode id 3)
-	  (im-update-mode id 0)))
+	  (im-update-mode id multi-segment-mode-wide-latin)
+	  (im-update-mode id multi-segment-mode-direct)))
   (canna-update-prop-label cc id))
 
 (define (canna-update-prop-list id)
@@ -828,6 +875,7 @@
 	   str
 	   "leaf\t\tҤ餬\tҤ餬ʤǤ\tprop_canna_hiragana\n"
 	   "leaf\t\t\tʤϤǤޤ\tprop_canna_katakana\n"
+	   "leaf\t\tȾѥ\tȾѥʤϤǤޤ\tprop_canna_hankana\n"
 	   "leaf\tA\tľ\t쥯ȤǤ\tprop_canna_direct\n"
 	   "leaf\t\tѱѿ\tѱѿ⡼\tprop_canna_zenkaku\n"
 	   "branch\t\t\nleaf\t\t޻\tdesc\tprop_canna_roma\n"
