@@ -161,6 +161,18 @@
 		 (im-update-preedit id)
 		 #f))
 	   #t)
+       (if (and (generic-prev-page-key? key state)
+		(<= generic-candidate-op-count (generic-context-candidate-op-count pc)))
+	   (begin
+	     (im-shift-page-candidate id #f)
+	     #f)
+	   #t)
+       (if (and (generic-next-page-key? key state)
+		(<= generic-candidate-op-count (generic-context-candidate-op-count pc)))
+	   (begin
+	     (im-shift-page-candidate id #t)
+	     #f)
+	   #t)
        (if (generic-backspace-key? key state)
 	   (begin
 	     (if (not (rk-backspace rkc))
@@ -238,8 +250,7 @@
   (lambda (id key state)
     (let* ((c (find-context id))
 	   (pc (context-data c)))
-      (if (and (number? key)
-	       (< key 32))
+      (if (control-char? key)
 	  (im-commit-raw id)
 	  (if (generic-context-on pc)
 	      (generic-proc-on-mode id pc key state)
@@ -249,11 +260,12 @@
 
 (define generic-key-release-handler
   (lambda (id key state)
-    (let* ((c (find-context id)))
-      (if (and (not (generic-context-on c))
-	       (and (number? key) (< key 32)))
-	  ;; don't eat key release event for apps
-	  (generic-commit-raw id)))))
+    (let* ((c (find-context id))
+	   (pc (context-data c)))
+      (if (or (control-char? key)
+	      (not (generic-context-on pc)))
+	  ;; don't discard key release event for apps
+	  (generic-commit-raw pc id)))))
 
 (define generic-reset-handler
   (lambda (id)
@@ -281,7 +293,10 @@
 
 (define generic-set-candidate-index-handler
   (lambda (id idx)
-    ()))
+    (let* ((c (find-context id))
+	   (pc (context-data c))
+	   (rkc (generic-context-rk-context pc)))
+      (generic-context-set-rk-nth! pc idx))))
 
 (define generic-init-handler
   (lambda (id init-handler)
