@@ -460,9 +460,16 @@
 
 ;(define ja-rk-rule ja-kana-rule)
 
+;; What's intended? The name "ja-direct-rule" makes me confused
+;; because it acts in non-direct modes. This rule makes
+;; uim-anthy improper as always sending zenkaku-space in
+;; hanakaku-kana mode. This seems to be used to "always commit
+;; zenkaku-space regardless input mode". If it's true, This rule
+;; should be able to be disabled. -- YamaKen
 (define ja-direct-rule
   '((" " "")
     ))
+
 ;;
 (define ja-find-rec
   (lambda (c rule)
@@ -488,6 +495,9 @@
     (require "japanese-kana.scm")
     (set! ja-rk-rule ja-kana-rule)
     (set! using-kana-table? #t)
+    (define-key anthy-kana-toggle-key? "")
+    (define-key anthy-latin-key? generic-on-key?)
+    (define-key anthy-wide-latin-key? "")
     ))
 
 (define load-azik-table
