@@ -1,5 +1,33 @@
-;
-(define global-case-insensitive-control-key? #f)
+;;;
+;;; Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
+;;;
+;;; All rights reserved.
+;;;
+;;; Redistribution and use in source and binary forms, with or without
+;;; modification, are permitted provided that the following conditions
+;;; are met:
+;;; 1. Redistributions of source code must retain the above copyright
+;;;    notice, this list of conditions and the following disclaimer.
+;;; 2. Redistributions in binary form must reproduce the above copyright
+;;;    notice, this list of conditions and the following disclaimer in the
+;;;    documentation and/or other materials provided with the distribution.
+;;; 3. Neither the name of authors nor the names of its contributors
+;;;    may be used to endorse or promote products derived from this software
+;;;    without specific prior written permission.
+;;;
+;;; THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+;;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+;;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+;;; ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+;;; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+;;; DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+;;; OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+;;; HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+;;; LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+;;; OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+;;; SUCH DAMAGE.
+;;;;
+
 ;
 (define shift-key-mask
   (lambda (state)
@@ -13,59 +41,62 @@
 (define meta-key-mask
   (lambda (state)
     (= (bit-and state 8) 8)))
+(define super-key-mask
+  (lambda (state)
+    (= (bit-and state 64) 64)))
+(define hyper-key-mask
+  (lambda (state)
+    (= (bit-and state 128) 128)))
 
 (define modifier-key-mask
   (lambda (state)
     (> state 0)))
 
-(define modifier-key
+(define modifier-key?
   (lambda (key key-state)
     (or
      (= key 'Shift_key)
      (= key 'Control_key)
      (= key 'Alt_key)
-     (= key 'Meta_key))))
+     (= key 'Meta_key)
+     (= key 'Super_key)
+     (= key 'Hyper_key))))
 
 (define key-state-alist
   '((Shift_key   . 1)
     (Control_key . 2)
     (Alt_key     . 4)
-    (Meta_key    . 8)))
+    (Meta_key    . 8)
+    (Super_key   . 64)
+    (Hyper_key   . 128)))
 
 (define emacs-like-prefix-alist
   '(("S" . Shift_key)
     ("C" . Control_key)
     ("A" . Alt_key)
     ("M" . Meta_key)
+    ("Z" . Super_key)
+    ("H" . Hyper_key)
     ("I" . IgnoreCase)
-    ("J" . IgnoreShift)))
+    ("J" . IgnoreShift)
+    ("K" . IgnoreRegularShift)))
 
 (define tag-prefix-alist
-  '(("Shift"       . Shift_key)
-    ("Control"     . Control_key)
-    ("Alt"         . Alt_key)
-    ("Meta"        . Meta_key)
-    ("IgnoreCase"  . IgnoreCase)
-    ("IgnoreShift" . IgnoreShift)))
+  '(("Shift"              . Shift_key)
+    ("Control"            . Control_key)
+    ("Alt"                . Alt_key)
+    ("Meta"               . Meta_key)
+    ("Super"              . Super_key)
+    ("Hyper"              . Hyper_key)
+    ("IgnoreCase"         . IgnoreCase)
+    ("IgnoreShift"        . IgnoreShift)
+    ("IgnoreRegularShift" . IgnoreRegularShift)))
 
-;;
-(define backspace-key
-  (lambda (key key-state)
-    (= key 'backspace)))
-(define left-key
-  (lambda (key key-state)
-    (= key 'left)))
-(define right-key
-  (lambda (key key-state)
-    (= key 'right)))
-
-(define switch-im-key
-  (lambda (key state)
-    (or
-     (and (control-key-mask state)
-	  (= key 'Shift_key))
-     (and (shift-key-mask state)
-	  (= key 'Control_key)))))
+(define translator-prefix?
+  (lambda (symbol)
+    (or (= symbol 'IgnoreCase)
+	(= symbol 'IgnoreShift)
+	(= symbol 'IgnoreRegularShift))))
 
 ;;
 (define intern-key-prefix
@@ -131,41 +162,47 @@
 
 (define parse-key-str
   (lambda (str translators key key-state)
-    (let ((str-len (string-length str))
-	  (rest ""))
+    (let ((str-len (string-length str)))
       (cond
        ((= str-len 0)
-	(list "" translators -1 key-state))
+	(list "" translators key key-state))
        ((= str-len 1)
 	(list "" translators (string->charcode str) key-state))
        ((<= 2 str-len)
 	(let* ((parsed (parse-key-prefix str))
-	       (prefix (car parsed)))
-	  (set! rest (cdr parsed))
+	       (prefix (car parsed))
+	       (rest (cdr parsed)))
 	  (cond
-	   ((modifier-key prefix 0)
-	    (set! key-state
-		  (bit-or key-state
-			  (cdr (assq prefix key-state-alist)))))
-	   ((= prefix 'IgnoreCase)
-	    (set! translators
-		  (cons (lambda (key key-state)
-			  (let ((translated-key (to-lower-char key)))
-			    (list translated-key key-state)))
-			translators)))
-	   ((= prefix 'IgnoreShift)
-	    (set! translators
-		  (cons (lambda (key key-state)
-			  (let ((translated-key-state (bit-and key-state
-							       (bit-not 1))))
-			    (list key translated-key-state)))
-			translators)))
+	   ((modifier-key? prefix 0)
+	    (let ((key-state (bit-or key-state
+				     (cdr (assq prefix key-state-alist)))))
+	      (parse-key-str rest translators key key-state)))
+	   ((translator-prefix? prefix)
+	    (let* ((translator
+		    (cond
+		     ((= prefix 'IgnoreCase)
+		      (lambda (key key-state)
+			(let ((translated-key (to-lower-char key)))
+			  (list translated-key key-state))))
+		     ((= prefix 'IgnoreShift)
+		      (lambda (key key-state)
+			(let ((translated-key-state (bit-and key-state
+                                                             (bit-not 1))))
+			  (list key translated-key-state))))
+		     ((= prefix 'IgnoreRegularShift)
+		      (lambda (key key-state)
+			(let ((translated-key-state (if (usual-char? key)
+							(bit-and key-state
+								 (bit-not 1))
+							key-state)))
+			  (list key translated-key-state))))))
+		   (translators (cons translator
+				      translators)))
+	      (parse-key-str rest translators key key-state)))
 	   (else
-	    (set! key (intern str))	; keysym
-	    (set! rest "")))
-	  (if (string-equal? rest "")
-	      (list rest translators key key-state)
-	      (parse-key-str rest translators key key-state))))))))
+	    (let ((key (intern str))	; keysym
+		  (rest ""))
+	      (parse-key-str rest translators key key-state))))))))))
 
 (define apply-translators
   (lambda (translators key key-state)
@@ -227,10 +264,7 @@
      ((string? key-strs)
       (let* ((key-str key-strs)
 	     (modified-key-str (string-append
-				"<IgnoreShift>"
-				(if global-case-insensitive-control-key?
-				    "<IgnoreCase>"
-				    "")
+				"<IgnoreRegularShift>"
 				key-str)))
 	modified-key-str))
      (else
@@ -254,3 +288,8 @@
 ;  (lambda (bind-name bind-table)
 ;    nil
 ;))
+
+;;
+(define-key left-key? "left")
+(define-key right-key? "right")
+(define-key switch-im-key? '("<Control>Shift_key" "<Shift>Control_key"))
