@@ -342,6 +342,7 @@
 	  
 	  (if (and
 	       prime-mask-pending-preedit
+	       left
 	       (usual-char? (string->charcode (car left))))
 	      (set! left (append (list "*") (cdr left))))
 	  
@@ -835,17 +836,18 @@
 (define prime-press-key-handler
   (lambda (id key state)
     (let* ((c (find-context id)))
-      (if (and (number? key)
-	       (< key 32))
+      (if (control-char? key)
 	  (im-commit-raw id)
 	  (prime-push-key c key state)))))
 
 (define prime-release-key-handler
   (lambda (id key state)
-    (if (and (number? key)
-	     (< key 32))
-	(im-commit-raw id)
-	())))
+    (let* ((c (find-context id))
+	   (sc (context-data c)))
+      (if (or (control-char? key)
+	      (= (prime-context-mode sc)
+		 prime-mode-direct))
+	  (im-commit-raw id)))))
 
 (define prime-reset-handler
   (lambda (id)
