@@ -123,7 +123,7 @@
 	   #t)
        (if (not tail-partial)
 	   (begin
-	     (set! res (cadr (rk-lib-find-seq longest-head rule)))
+	     (set! res (cadr (rk-lib-find-seq (reverse longest-head) rule)))
 	     (rk-context-set-seq!
 	      rkc
 	      (reverse
