@@ -659,6 +659,7 @@
 	   #t)
        (if (or
 	    (control-key-mask key-state)
+	    (alt-key-mask key-state)
 	    (= key 32))
 	   (begin
 	     (skk-commit-raw sc id key key-state)
@@ -1072,7 +1073,7 @@
 (define skk-press-key-handler
   (lambda (id key state)
     (let ((c (find-context id)))
-      (if (and (number? key) (< key 32))
+      (if (control-char? key)
 	  (im-commit-raw id)
 	  (skk-push-key c id key state)))))
 
@@ -1082,7 +1083,7 @@
 	   (sc (skk-current-context c))
 	   (state (skk-context-state sc)))
       (if (= state 'skk-state-latin)
-	  ;; don't eat key release event for apps
+	  ;; don't discard key release event for apps
 	  (begin
 	    (skk-context-set-commit-raw! sc #f)
 	    (im-commit-raw id))))))
