@@ -29,6 +29,11 @@
 ;;; SUCH DAMAGE.
 ;;;;
 
+;;
+(define control-char?
+  (lambda (c)
+    (and (number? c)
+	 (< c 32))))
 ;;
 (define alphabet-char?
   (lambda (c)
@@ -160,3 +165,55 @@
     (separator-background          . "")
     (reversed-separator-foreground . "black")
     (reversed-separator-background . "blue")))
+
+;; functions for multiple segments input method
+(define multi-segment-type-hiragana 0)
+(define multi-segment-type-katakana 1)
+(define multi-segment-type-hankana 2)
+
+(define multi-segment-mode-direct 0)
+(define multi-segment-mode-hiragana 1)
+(define multi-segment-mode-katakana 2)
+(define multi-segment-mode-wide-latin 3)
+(define multi-segment-mode-hankana 4)
+
+(define multi-segment-make-index-list
+  (lambda (n old-lst)
+    (if (< n (length old-lst))
+	(truncate-list old-lst n 1)
+	(append old-lst
+		(multi-segment-make-index-list-rec (- n
+					(length old-lst)))))))
+(define multi-segment-make-index-list-rec
+  (lambda (n)
+    (if (> n 0)
+	(cons 0
+	      (multi-segment-make-index-list-rec (- n 1)))
+	'())))
+
+(define multi-segment-make-string
+  (lambda (sl dir type)
+    (let ((get-str-by-type
+	   (lambda (l)
+	     (cond
+	      ((= type multi-segment-type-hiragana)
+	       (caar l))
+	      ((= type multi-segment-type-katakana)
+	       (car (cdar l)))
+	      ((= type multi-segment-type-hankana)
+	       (cadr (cdar l)))))))
+      (if sl
+	  (if dir
+	      (string-append (multi-segment-make-string (cdr sl) dir type)
+			     (get-str-by-type sl))
+	      (string-append (get-str-by-type sl)
+			     (multi-segment-make-string (cdr sl) dir type)))
+	  ""))))
+
+(define multi-segment-make-left-string
+  (lambda (sl kana)
+    (multi-segment-make-string sl #t kana)))
+
+(define multi-segment-make-right-string
+  (lambda (sl kana)
+    (multi-segment-make-string sl #f kana)))
