@@ -190,8 +190,8 @@ tty2key(char *keybuf, int i)
     return 0; 
     
   } else {
-    if (keybuf[0] >= 0x41 && keybuf[0] <= 0x5a)
-      return keybuf[0] + ('a' - 'A');
+    if (keybuf[0] >= 0x01 && keybuf[0] <= 0x1a)
+     return keybuf[0] + ('a' - 0x01);
     switch (keybuf[0]) {
     case 0x7f:
       return UKey_Backspace;
@@ -256,13 +256,10 @@ main(int argc, char **argv)
     key = tty2key(keybuf, i);
     mod = tty2mod(keybuf, i);
 
-    if (key == UKey_Zenkaku_Hankaku)
-      mod = 0;
-
     rf = uim_press_key(ac->uc, key, mod);
     uim_release_key(ac->uc, key, mod);
     if (rf) {
-      printf("%c",keybuf[0]);
+      printf("%s",keybuf);
     }
   }
   return 0;
