@@ -1,30 +1,7 @@
 /*
   canna.c: Canna for uim.
 
-  Copyright (c) 2003,2004 Anthy Project http://anthy.sourceforge.jp/
   Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
-  Copyright (c) 2003,2004 Masahito Omote <omote@utyuuzin.net>
-
-  You can use, modify, distribute under the term of GPL or BSD Licence.
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
-  GNU Library General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-
-
-  Copyright (c) 2003,2004 Anthy Project http://anthy.sourceforge.jp/
-  Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
-  Copyright (c) 2003,2004 Masahito Omote <omote@utyuuzin.net>
 
   All rights reserved.
 
