@@ -84,7 +84,7 @@ struct uim_context_ {
   int candidate_index;
   int nr_candidates;
   /**/
-  void (*commit_cb)(void *ptr, char *str);
+  void (*commit_cb)(void *ptr, const char *str);
   /* preedit */
   void (*preedit_clear_cb)(void *ptr);
   void (*preedit_pushback_cb)(void *ptr, int attr, char *str);
@@ -94,9 +94,9 @@ struct uim_context_ {
   /* mode */
   void (*mode_update_cb)(void *ptr, int);
   /* property list */
-  void (*prop_list_update_cb)(void *ptr, char *str);
+  void (*prop_list_update_cb)(void *ptr, const char *str);
   /* property label */
-  void (*prop_label_update_cb)(void *ptr, char *str);
+  void (*prop_label_update_cb)(void *ptr, const char *str);
   /* candidate window */
   void (*candidate_selector_activate_cb)(void *ptr, int nr, int index);
   void (*candidate_selector_select_cb)(void *ptr, int index);
