@@ -183,14 +183,6 @@ typedef LISP (*SUBR_FUNC) (void);
 
 #define TKBUFFERN 5120
 
-struct repl_hooks
-  {
-    void (*repl_puts) (char *);
-      LISP (*repl_read) (void);
-      LISP (*repl_eval) (LISP);
-    void (*repl_print) (LISP);
-  };
-
 void siod_init (int argc, char **argv, int warnflag, FILE *);
 void siod_quit (void);
 
@@ -198,22 +190,18 @@ void set_repl_hooks (void (*puts_f) (char *),
 		     LISP (*read_f) (void),
 		     LISP (*eval_f) (LISP),
 		     void (*print_f) (LISP));
-LISP my_err (char *message, LISP x);
 char *get_c_string (LISP x);
 char *get_c_string_dim (LISP x, long *);
 long get_c_long (LISP x);
 long nlength(LISP x);
 
 LISP cons (LISP x, LISP y);
-LISP consp (LISP x);
 LISP car (LISP x);
 LISP cdr (LISP x);
 LISP setcar (LISP cell, LISP value);
 LISP intcons (int x);
-LISP numberp (LISP x);
 LISP eql (LISP x, LISP y);
 LISP symcons (char *pname, LISP vcell);
-LISP symbolp (LISP x);
 LISP symbol_boundp (LISP x, LISP env);
 LISP symbol_value (LISP x, LISP env);
 LISP rintern (const char *name);
@@ -242,11 +230,7 @@ LISP lread (LISP);
 LISP lreadtk (char *, long j);
 LISP lreadf (FILE * f);
 LISP require (LISP fname);
-LISP nullp (LISP x);
 LISP strcons (long length, char *data);
-//LISP copy_list_slib (LISP);
-LISP href (LISP, LISP);
-LISP hset (LISP, LISP, LISP);
 LISP equal (LISP, LISP);
 void set_fatal_exit_hook (void (*fcn) (void));
 LISP intern (LISP x);
@@ -267,14 +251,12 @@ LISP cddar (LISP);
 LISP cdddr (LISP);
 LISP siod_true_value (void);
 LISP lapply (LISP fcn, LISP args);
-LISP mallocl (void *lplace, long size);
-void put_st (char *st);
 LISP listn (long n,...);
 char *must_malloc (unsigned long size);
 FILE *get_c_file (LISP p, FILE * deflt);
 char *last_c_errmsg (int);
 LISP llast_c_errmsg (int);
-void c_provide(char *);
+void siod_c_provide(const char *);
 
 LISP funcall1 (LISP, LISP);
 LISP funcall2 (LISP, LISP, LISP);
@@ -285,20 +267,8 @@ LISP lputc (LISP c, LISP p);
 LISP lputs (LISP str, LISP p);
 
 extern long siod_verbose_level;
-extern char *siod_lib;
-
-struct gc_protected
-  {
-    LISP *location;
-    long length;
-    struct gc_protected *next;
-  };
-
-#define INTERRUPT_CHECK()
+void siod_set_lib_path(const char *);
 
 long no_interrupt (long n);
-void fput_st (FILE * f, char *st);
-void put_st (char *st);
-void gc_for_newcell (void);
 
 #endif /* __SIOD_H__ */
