@@ -91,6 +91,8 @@
 
 #include "siod.h"
 
+/* strcut */
+
 struct catch_frame
   {
     LISP tag;
@@ -114,12 +116,6 @@ struct gen_printio
     void *cb_argument;
   };
 
-#define GETC_FCN(x) (*((*x).getc_fcn))((*x).cb_argument)
-#define UNGETC_FCN(c,x) (*((*x).ungetc_fcn))(c,(*x).cb_argument)
-#define PUTC_FCN(c,x) (*((*x).putc_fcn))(c,(*x).cb_argument)
-#define PUTS_FCN(c,x) (*((*x).puts_fcn))(c,(*x).cb_argument)
-
-
 struct user_type_hooks
   {
       LISP (*gc_mark) (LISP);
@@ -129,6 +125,33 @@ struct user_type_hooks
       LISP (*equal) (LISP, LISP);
   };
 
+struct repl_hooks
+  {
+    void (*repl_puts) (char *);
+      LISP (*repl_read) (void);
+      LISP (*repl_eval) (LISP);
+    void (*repl_print) (LISP);
+  };
+
+
+struct gc_protected
+  {
+    LISP *location;
+    long length;
+    struct gc_protected *next;
+  };
+
+/* forward declaration of static functions */
+static void gc_for_newcell (void);
+
+/* macros */
+
+#define INTERRUPT_CHECK()
+
+#define GETC_FCN(x) (*((*x).getc_fcn))((*x).cb_argument)
+#define UNGETC_FCN(c,x) (*((*x).ungetc_fcn))(c,(*x).cb_argument)
+#define PUTC_FCN(c,x) (*((*x).putc_fcn))(c,(*x).cb_argument)
+#define PUTS_FCN(c,x) (*((*x).puts_fcn))(c,(*x).cb_argument)
 
 #define STACK_LIMIT(_ptr,_amt) (((char *)_ptr) - (_amt))
 
@@ -214,6 +237,76 @@ static LISP lreadr (struct gen_readio *f);
 
 #define ENVLOOKUP_TRICK 1
 
+static long inside_err = 0;
+
+static LISP
+my_err (char *message, LISP x)
+{
+  struct catch_frame *l;
+  long was_inside = inside_err;
+  LISP retval, nx;
+  char *msg, *eobj;
+  nointerrupt = 1;
+  if ((!message) && CONSP (x) && TYPEP (CAR (x), tc_string))
+    {
+      msg = get_c_string (CAR (x));
+      nx = CDR (x);
+      retval = x;
+    }
+  else
+    {
+      msg = message;
+      nx = x;
+      retval = NIL;
+    }
+  if ((eobj = try_get_c_string (nx)) && !memchr (eobj, 0, 30))
+    eobj = NULL;
+
+  if NULLP
+    (nx)
+      sprintf (siod_err_msg, "ERROR: %s\n", msg);
+  else if (eobj)
+    sprintf (siod_err_msg, "ERROR: %s (errobj %s)\n", msg, eobj);
+  else
+    sprintf (siod_err_msg, "ERROR: %s (see errobj)\n", msg);
+
+  if ((siod_verbose_level >= 1) && msg)
+    {
+      fprintf (siod_output, "%s\n", siod_err_msg);
+      fflush (siod_output);
+    }
+  if (errjmp_ok == 1)
+    {
+      inside_err = 1;
+      setvar (sym_errobj, nx, NIL);
+      for (l = catch_framep; l; l = (*l).next)
+	if (EQ ((*l).tag, sym_errobj) ||
+	    EQ ((*l).tag, sym_catchall))
+	  {
+	    if (!msg)
+	      msg = "quit";
+	    (*l).retval = (NNULLP (retval) ? retval :
+			   (was_inside) ? NIL :
+			   cons (strcons (strlen (msg), msg), nx));
+	    nointerrupt = 0;
+	    inside_err = 0;
+	    longjmp ((*l).cframe, 2);
+	  }
+      inside_err = 0;
+      longjmp (errjmp, (msg) ? 1 : 2);
+    }
+  if (siod_verbose_level >= 1)
+    {
+      fprintf (stderr, "FATAL ERROR DURING STARTUP OR CRITICAL CODE SECTION\n");
+      fflush (stderr);
+    }
+  if (fatal_exit_hook)
+    (*fatal_exit_hook) ();
+  else
+    exit (1);
+  return (NIL);
+}
+
 static LISP
 envlookup (LISP var, LISP env)
 {
@@ -353,6 +446,29 @@ myrealtime (void)
   return ((double) x);
 }
 
+
+static void
+fput_st (FILE * f, char *st)
+{
+  long flag;
+  flag = no_interrupt (1);
+  if (siod_verbose_level >= 1)
+    {
+      //      fprintf (stdout, "%s", st);
+      //      fflush (siod_output);
+      fprintf (f, "%s", st);
+      fflush (f);
+    }
+  no_interrupt (flag);
+}
+
+static void
+put_st (char *st)
+{
+  fput_st (siod_output, st);
+  fflush (siod_output);
+}
+
 static void
 grepl_puts (char *st, void (*repl_puts) (char *))
 {
@@ -509,6 +625,12 @@ lprin1 (LISP exp, LISP lf)
   return (NIL);
 }
 
+void
+siod_set_lib_path(const char *path)
+{
+  siod_lib = path;
+}
+
 static LISP
 vload (char *fname, long cflag, long rflag)
 {
@@ -1035,21 +1157,6 @@ gput_st (struct gen_printio *f, char *st)
   PUTS_FCN (st, f);
 }
 
-void
-fput_st (FILE * f, char *st)
-{
-  long flag;
-  flag = no_interrupt (1);
-  if (siod_verbose_level >= 1)
-    {
-      //      fprintf (stdout, "%s", st);
-      //      fflush (siod_output);
-      fprintf (f, "%s", st);
-      fflush (f);
-    }
-  no_interrupt (flag);
-}
-
 int
 fputs_fcn (char *st, void *cb)
 {
@@ -1057,21 +1164,12 @@ fputs_fcn (char *st, void *cb)
   return (1);
 }
 
-void
-put_st (char *st)
-{
-  fput_st (siod_output, st);
-  fflush (siod_output);
-}
-
 void
 set_fatal_exit_hook (void (*fcn) (void))
 {
   fatal_exit_hook = fcn;
 }
 
-static long inside_err = 0;
-
 
 static char *
 try_get_c_string (LISP x)
@@ -1086,75 +1184,6 @@ try_get_c_string (LISP x)
     return (NULL);
 }
 
-LISP
-my_err (char *message, LISP x)
-{
-  struct catch_frame *l;
-  long was_inside = inside_err;
-  LISP retval, nx;
-  char *msg, *eobj;
-  nointerrupt = 1;
-  if ((!message) && CONSP (x) && TYPEP (CAR (x), tc_string))
-    {
-      msg = get_c_string (CAR (x));
-      nx = CDR (x);
-      retval = x;
-    }
-  else
-    {
-      msg = message;
-      nx = x;
-      retval = NIL;
-    }
-  if ((eobj = try_get_c_string (nx)) && !memchr (eobj, 0, 30))
-    eobj = NULL;
-
-  if NULLP
-    (nx)
-      sprintf (siod_err_msg, "ERROR: %s\n", msg);
-  else if (eobj)
-    sprintf (siod_err_msg, "ERROR: %s (errobj %s)\n", msg, eobj);
-  else
-    sprintf (siod_err_msg, "ERROR: %s (see errobj)\n", msg);
-
-  if ((siod_verbose_level >= 1) && msg)
-    {
-      fprintf (siod_output, "%s\n", siod_err_msg);
-      fflush (siod_output);
-    }
-  if (errjmp_ok == 1)
-    {
-      inside_err = 1;
-      setvar (sym_errobj, nx, NIL);
-      for (l = catch_framep; l; l = (*l).next)
-	if (EQ ((*l).tag, sym_errobj) ||
-	    EQ ((*l).tag, sym_catchall))
-	  {
-	    if (!msg)
-	      msg = "quit";
-	    (*l).retval = (NNULLP (retval) ? retval :
-			   (was_inside) ? NIL :
-			   cons (strcons (strlen (msg), msg), nx));
-	    nointerrupt = 0;
-	    inside_err = 0;
-	    longjmp ((*l).cframe, 2);
-	  }
-      inside_err = 0;
-      longjmp (errjmp, (msg) ? 1 : 2);
-    }
-  if (siod_verbose_level >= 1)
-    {
-      fprintf (stderr, "FATAL ERROR DURING STARTUP OR CRITICAL CODE SECTION\n");
-      fflush (stderr);
-    }
-  if (fatal_exit_hook)
-    (*fatal_exit_hook) ();
-  else
-    exit (1);
-  return (NIL);
-}
-
-
 static LISP
 last (LISP l)
 {
@@ -1258,7 +1287,7 @@ cons (LISP x, LISP y)
   return (z);
 }
 
-LISP
+static LISP
 consp (LISP x)
 {
   if CONSP
@@ -1320,7 +1349,7 @@ intcons (int x)
   return (z);
 }
 
-LISP
+static LISP
 numberp (LISP x)
 {
   if INTNUMP
@@ -1606,7 +1635,7 @@ symcons (char *pname, LISP vcell)
   return (z);
 }
 
-LISP
+static LISP
 symbolp (LISP x)
 {
   if SYMBOLP
@@ -2308,7 +2337,7 @@ gc_mark_and_sweep (void)
   gc_ms_stats_end ();
 }
 
-void
+static void
 gc_for_newcell (void)
 {
   long flag, n;
@@ -3399,7 +3428,7 @@ provide (LISP name)
 }
 
 void
-c_provide(char *name)
+siod_c_provide(const char *name)
 {
   provide(rintern(name));
 }
@@ -3439,7 +3468,7 @@ quit (void)
   return (my_err (NULL, NIL));
 }
 
-LISP
+static LISP
 nullp (LISP x)
 {
   if EQ
