@@ -54,19 +54,18 @@
 #endif
 
 static int context_id = 0;
-static int focused_context_id;
 
+static int nr_client_slots;
 static struct client {
   int fd;
   char *rbuf;
-  int rbuf_count;
-  int rerr;
-} clients[MAX_CLIENT];
+} *clients;
 
 /*
   prepare file descriptor.
 */
-static int init_serv_fd(char *path)
+static int
+init_serv_fd(char *path)
 {
   int foo;
   int fd;
@@ -81,8 +80,8 @@ static int init_serv_fd(char *path)
   myhost.sun_family = PF_UNIX;
   strcpy(myhost.sun_path, path);
 
-  foo = bind(fd,(struct sockaddr *)&myhost,SUN_LEN(&myhost));
-  if(foo < -1){
+  foo = bind(fd, (struct sockaddr *)&myhost, SUN_LEN(&myhost));
+  if (foo < -1) {
     perror("failed in bind()");
     return -1;
   }
@@ -90,7 +89,7 @@ static int init_serv_fd(char *path)
   chmod(path, S_IRUSR|S_IWUSR);
 
   foo = listen(fd, SOMAXCONN);
-  if ( foo == -1){
+  if (foo == -1) {
     perror("failed in listen()");
     return -1;
   }
@@ -98,36 +97,28 @@ static int init_serv_fd(char *path)
   return fd;
 }
 
-static void
-init_client_array()
-{
-  int i;
-  for (i = 0; i < MAX_CLIENT; i++) {
-    clients[i].fd = -1;
-    clients[i].rbuf = strdup("");
-  }
-}
-
 static struct client *
 get_unused_client()
 {
   int i;
-  for (i = 0; i < MAX_CLIENT; i++) {
+  for (i = 0; i < nr_client_slots; i++) {
     if (clients[i].fd == -1) {
-      clients[i].rbuf_count = 0;
-      clients[i].rerr = 0;
-      if (!clients[i].rbuf) {
-	clients[i].rbuf = malloc(BUFFER_SIZE);
-      }
       return &clients[i];
     }
   }
-  return NULL;
+  nr_client_slots ++;
+  clients = realloc(clients, sizeof(struct client) * nr_client_slots);
+  clients[nr_client_slots - 1].rbuf = strdup("");
+  return &clients[nr_client_slots - 1];
 }
 
 static void
 free_client(struct client *cl)
 {
+  if (cl->rbuf) {
+    free(cl->rbuf);
+    cl->rbuf = strdup("");
+  }
   cl->fd = -1;
 }
 
@@ -137,20 +128,19 @@ parse_content(char *content, struct client *cl)
 {
   int i;
 
-  for (i = 0; i < MAX_CLIENT; i ++){
+  for (i = 0; i < nr_client_slots; i ++){
     if(clients[i].fd != -1 && 
        clients[i].fd != cl->fd && 
-       uim_helpler_fd_writable(clients[i].fd))
+       uim_helpler_fd_writable(clients[i].fd)) {
       write(clients[i].fd, content, strlen(content));
+    }
   }
-
-    
-    //    write(cl->fd, buf, strlen(buf));
 }
 
-int proc_func(struct client *cl)
+int
+proc_func(struct client *cl)
 {
-  int i, rc, count;
+  int i, rc;
   char buf[BUFFER_SIZE];
 
   /* do read */
@@ -169,7 +159,7 @@ int proc_func(struct client *cl)
   cl->rbuf = (char*)realloc(cl->rbuf, strlen(cl->rbuf) + strlen(buf)+1);
   strcat(cl->rbuf, buf);
 
-  if(uim_helper_str_terminated(cl->rbuf)) {
+  if (uim_helper_str_terminated(cl->rbuf)) {
     /* process */
     parse_content(cl->rbuf, cl);
     free(cl->rbuf);
@@ -188,17 +178,16 @@ uim_helper_server_process_connection(int serv_fd)
 
   fd_biggest = serv_fd;
 
-  while(1) {
-
+  while (1) {
     /* setup readfds */
     FD_ZERO(&readfds);
     FD_SET(serv_fd, &readfds);  
-    for (i = 0; i < MAX_CLIENT; i ++) {
+    for (i = 0; i < nr_client_slots; i ++) {
       int fd = clients[i].fd;
       if (fd == -1) {
 	continue;
       }
-      FD_SET(clients[i].fd, &readfds);
+      FD_SET(fd, &readfds);
       if (fd > fd_biggest) {
 	fd_biggest = fd;
       }
@@ -237,7 +226,7 @@ uim_helper_server_process_connection(int serv_fd)
     }    
     
     /* check data from clients reached */
-    for (i = 0; i < MAX_CLIENT; i ++) {
+    for (i = 0; i < nr_client_slots; i ++) {
       if (clients[i].fd != -1 &&
 	  FD_ISSET(clients[i].fd, &readfds)) {
 	int result;
@@ -259,7 +248,9 @@ main(int argc, char **argv)
   char *path = uim_helper_get_pathname();
   int serv_fd;
   unlink(path);
-  init_client_array();
+
+  clients = NULL;
+  nr_client_slots = 0;
 
   serv_fd = init_serv_fd(path);
 
