@@ -190,14 +190,14 @@ uim_ipc_send_command(int *pid, FILE **read_handler,
   char buf[8192];
 
   if (*read_handler == NULL || *write_handler == NULL) {
-    *pid = uim_ipc_open_command(*pid, read_handler, write_handler, command );
+    *pid = uim_ipc_open_command(*pid, read_handler, write_handler, command);
   }
   if(*pid == 0) {
     free(tmp);
     return NULL;
   }
 
-  fprintf( *write_handler, str );
+  fputs(str, *write_handler);
 
  again:
   if (fflush(*write_handler) != 0) {
