@@ -32,6 +32,7 @@
 */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include "context.h"
 
 static struct {
@@ -194,8 +195,8 @@ handle_key(uim_context uc, char *p, int key, int state)
 static int
 emergency_key_p(int key, int state)
 {
-  if ((state == UMod_Control) &&
-      (key == UKey_Shift_key)) {
+  if ((state == UMod_Shift) &&
+      (key == UKey_Backspace)) {
     return 1;
   }
   return 0;
@@ -207,7 +208,7 @@ uim_press_key(uim_context uc, int key, int state)
   if (!uc) {
     return 1;
   }
-  if (emergency_key_p(key, state)) {
+  if (getenv("LIBUIM_ENABLE_EMERGENCY_KEY") && emergency_key_p(key, state)) {
     uc->is_enable = uc->is_enable ? 0 : 1;
     return 0;
   }
