@@ -44,7 +44,6 @@
 
 
 extern long siod_verbose_level;
-extern char *siod_lib;
 extern char *uim_return_str;
 extern char *uim_return_str_list[10];
 /* duplicate definition */
@@ -101,11 +100,11 @@ put_context_id(uim_context is)
 
 uim_context
 uim_create_context(void *ptr,
-		   char *enc,
-		   char *lang,
-		   char *engine,
+		   const char *enc,
+		   const char *lang,
+		   const char *engine,
 		   struct uim_code_converter *conv,
-		   void (*commit_cb)(void *ptr, char *str))
+		   void (*commit_cb)(void *ptr, const char *str))
 {
   uim_context uc;
   int buf_len;
@@ -242,7 +241,7 @@ uim_set_mode_list_update_cb(uim_context uc,
 
 void
 uim_set_prop_list_update_cb(uim_context uc,
-			    void (*update_cb)(void *ptr, char *str))
+			    void (*update_cb)(void *ptr, const char *str))
 {
   uc->prop_list_update_cb = update_cb;
 }
@@ -250,7 +249,7 @@ uim_set_prop_list_update_cb(uim_context uc,
 
 void
 uim_set_prop_label_update_cb(uim_context uc,
-			     void (*update_cb)(void *ptr, char *str))
+			     void (*update_cb)(void *ptr, const char *str))
 {
   uc->prop_label_update_cb = update_cb;
 }
@@ -524,9 +523,9 @@ uim_init_scm()
 
   scm_files = getenv("LIBUIM_SCM_FILES");
   if (scm_files) {
-    siod_lib = scm_files;
+    siod_set_lib_path(scm_files);
   } else {
-    siod_lib = SCM_FILES;
+    siod_set_lib_path(SCM_FILES);
   }
 
   load_file("im.scm");
