@@ -194,22 +194,25 @@ enum UPreeditAttr {
 
 /* abstracting platform-dependent character code conversion method */
 struct uim_code_converter {
-  int (*is_convertible)(const char *tocode, const char *fromcode);
+  int  (*is_convertible)(const char *tocode, const char *fromcode);
   void *(*create)(const char *tocode, const char *fromcode);
   char *(*convert)(void *obj, const char *str);
   void (*release)(void *obj);
 };
   
-int uim_init(void);
-void uim_quit(void);
+int
+uim_init (void);
+
+void
+uim_quit (void);
 
 uim_context
 uim_create_context(void *ptr,
-		   char *enc,
-		   char *lang,
-		   char *engine,
+		   const char *enc,
+		   const char *lang,
+		   const char *engine,
 		   struct uim_code_converter *conv,
-		   void (*commit_cb)(void *ptr, char *str));
+		   void (*commit_cb)(void *ptr, const char *str));
 void
 uim_set_preedit_cb(uim_context uc,
 		   void (*clear_cb)(void *ptr),
@@ -251,12 +254,11 @@ uim_prop_list_update(uim_context uc);
 void
 uim_prop_label_update(uim_context uc);
 void
-uim_set_prop_list_update_cb(uim_context uc,
-			    void (*update_cb)(void *ptr, char *str));
+uim_set_prop_list_update_cb(uim_context uc, void (*update_cb)(void *ptr, const char *str));
 
 void
 uim_set_prop_label_update_cb(uim_context uc,
-			     void (*update_cb)(void *ptr, char *str));
+			     void (*update_cb)(void *ptr, const char *str));
 void
 uim_prop_activate(uim_context uc, const char *str);
 /* mode */
