@@ -407,7 +407,9 @@ helper_str_parse(char *str)
 	    }
 
 	    uim_prop_activate(focusedContext->getUC(), line);
-	}
+	} else if (strncmp("focus_in", line, 8) == 0) {
+            InputContext::deletefocusedContext();
+ 	}
     }
 }
 
