@@ -231,7 +231,7 @@ public:
     void changeMode(int );
     void setICAttrs(void *,int );
     int get_ic_atr(int ,TxPacket *);
-    void commit_string(char *s);
+    void commit_string(const char *s);
     void extra_input(char *t);
     void reset_ic();
     Convdisp *get_convdisp();
