@@ -362,12 +362,11 @@ int XimIC::get_imid()
 
 void XimIC::setFocus()
 {
-    if (mIsActive) {
-	return ;
+    if (!mIsActive) {
+	nrActiveIC ++; 
     }
     current_ic = this;
     mIsActive = true;
-    nrActiveIC ++;
     ui_update_ic_stat();
     ui_update_input_mode(m_kkContext->getMode());
     m_kkContext->focusIn();
@@ -442,7 +441,7 @@ void XimIC::send_key_event(XKeyEvent *e)
     mConn->push_packet(t);
 }
 
-void XimIC::commit_string(char *str)
+void XimIC::commit_string(const char *str)
 {
     uString us;
     mConn->getXimServer()->strToUstring(&us, str);
