@@ -216,7 +216,7 @@ XimServer::uStringToCtext(uString *us)
 }
 
 void
-XimServer::strToUstring(uString *d, char *s)
+XimServer::strToUstring(uString *d, const char *s)
 {
     int i,len;
     bool inJis= false;
@@ -306,6 +306,12 @@ InputContext::focusedContext()
     return mFocusedContext;
 }
 
+void
+InputContext::deletefocusedContext()
+{
+    mFocusedContext = NULL;
+}
+
 void
 InputContext::focusIn()
 {
@@ -334,7 +340,7 @@ InputContext::getUC()
 }
 
 void
-InputContext::commit_cb(void *ptr, char *str)
+InputContext::commit_cb(void *ptr, const char *str)
 {
     InputContext *ic = (InputContext *)ptr;
     XimIC *xic = ic->get_ic();
@@ -380,13 +386,13 @@ void InputContext::candidate_deactivate_cb(void *ptr)
     ic->candidate_deactivate();
 }
 
-void InputContext::update_prop_list_cb(void *ptr, char *str)
+void InputContext::update_prop_list_cb(void *ptr, const char *str)
 {
     InputContext *ic = (InputContext *)ptr;
     ic->update_prop_list(str);
 }
 
-void InputContext::update_prop_label_cb(void *ptr, char *str)
+void InputContext::update_prop_label_cb(void *ptr, const char *str)
 {
     InputContext *ic = (InputContext *)ptr;
     ic->update_prop_label(str);
@@ -508,7 +514,7 @@ void InputContext::candidate_deactivate()
     disp->deactivate();
 }
 
-void InputContext::update_prop_list(char* str)
+void InputContext::update_prop_list(const char* str)
 {
     char tmp[BUFSIZ];
     int len;
@@ -521,7 +527,7 @@ void InputContext::update_prop_list(char* str)
     }
 }
 
-void InputContext::update_prop_label(char* str)
+void InputContext::update_prop_label(const char* str)
 {
     char tmp[BUFSIZ];
     int len;
