@@ -160,19 +160,20 @@ public:
     void candidate_activate(int nr, int display_limit);
     void candidate_select(int index);
     void candidate_deactivate();
-    void update_prop_list(char *str);
-    void update_prop_label(char *str);
+    void update_prop_list(const char *str);
+    void update_prop_label(const char *str);
 public:
-    static void commit_cb(void *, char *);
+    static void commit_cb(void *, const char *);
     static void clear_cb(void *);
     static void pushback_cb(void *, int attr, char *str);
     static void update_cb(void *);
     static void candidate_activate_cb(void *ptr, int nr, int index);
     static void candidate_select_cb(void *ptr, int index);
     static void candidate_deactivate_cb(void *ptr);
-    static void update_prop_list_cb(void *ptr, char *str);
-    static void update_prop_label_cb(void *ptr, char *str);
+    static void update_prop_list_cb(void *ptr, const char *str);
+    static void update_prop_label_cb(void *ptr, const char *str);
     static InputContext *focusedContext();
+    static void deletefocusedContext();
 protected:
     void commit_string(char *);
     XimIC *mXic;
@@ -202,7 +203,7 @@ public:
     bool setupConnection();
     void setupNewConnection(XClientMessageEvent *ev);
     char *uStringToCtext(uString *js);
-    void strToUstring(uString *d, char *s);
+    void strToUstring(uString *d, const char *s);
     struct input_style *getInputStyles();
 public:
     static XimServer *findServer(Window w);
